@@ -4,8 +4,10 @@ require_once dirname(__FILE__)."/accesscheck.php";
 mt_srand((double)microtime()*1000000);
 $randval = mt_rand();
 
-if (!$id) {
+if (empty($id) && isset($_GET['id'])) {
   $id = sprintf('%d',$_GET["id"]);
+} elseif (!isset($id)) {
+  $id = 0;
 }
 
 if (!$id && $_GET["page"] != "import1") {
@@ -15,12 +17,21 @@ if (!$id && $_GET["page"] != "import1") {
 require_once dirname(__FILE__)."/date.php";
 $date = new Date();
 
+## Check if input is complete
 $allthere = 1;
 $subscribepagedata = PageData($id);
-if (isset($subscribepagedata['language_file']) && is_file(dirname(__FILE__).'/../texts/'.$subscribepagedata['language_file'])) {
-  @include dirname(__FILE__).'/../texts/'.$subscribepagedata['language_file'];
+if (isset($subscribepagedata['language_file']) && is_file(dirname(__FILE__).'/../texts/'.basename($subscribepagedata['language_file']))) {
+  @include_once dirname(__FILE__).'/../texts/'.basename($subscribepagedata['language_file']);
 }
-$required = array();
+# Allow customisation per installation
+if (is_file($_SERVER['DOCUMENT_ROOT'].'/'.basename($GLOBALS["language_module"]))) {
+  include_once $_SERVER['DOCUMENT_ROOT'].'/'.basename($GLOBALS["language_module"]);
+}
+if (!empty($data['language_file']) && is_file($_SERVER['DOCUMENT_ROOT'].'/'.basename($data['language_file']))) {
+  include_once $_SERVER['DOCUMENT_ROOT'].'/'.basename($data['language_file']);
+}
+
+$required = array();   # id's of missing attribbutes
 if (sizeof($subscribepagedata)) {
   $attributes = explode('+',$subscribepagedata["attributes"]);
   foreach ($attributes as $attribute) {
@@ -52,7 +63,7 @@ if (sizeof($required)) {
       $thisonemissing = 0;
       if ($row["type"] != "hidden") {
         $thisonemissing = empty($_POST[$fieldname]);
-        if ($thisonemissing && isset($_POST[$fieldname]))
+        if ($thisonemissing)
           $missing .= $row["name"] .", ";
         $allthere = $allthere && !$thisonemissing;
       }
@@ -75,6 +86,20 @@ if (isset($subscribepagedata['emaildoubleentry']) && $subscribepagedata['emaildo
     $missing = $GLOBALS["strEmailsNoMatch"];
   }
 }
+
+// anti spambot check
+if (!empty($_POST['VerificationCodeX'])) {
+  if (NOTIFY_SPAM) {
+    $msg = $GLOBALS['I18N']->get('spamblockemailintro');
+    foreach ($_REQUEST as $key => $val) {
+      $msg .= "\n".'Form field: '.htmlentities($key)."\n".'================='."\nSubmitted value: ".htmlentities($val)."\n".'=============='."\n\n";
+    }
+    sendAdminCopy("phplist Spam blocked","\n".$msg);
+  }
+  unset($msg);
+  return;
+}
+
 if (!isset($_POST['passwordreq'])) $_POST['passwordreq'] = '';
 if (!isset($_POST['password'])) $_POST['password'] = '';
 
@@ -104,223 +129,280 @@ if (isset($_POST["email"]) && $check_for_host) {
 
 $listsok = ((!ALLOW_NON_LIST_SUBSCRIBE && isset($_POST["list"]) && is_array($_POST["list"])) || ALLOW_NON_LIST_SUBSCRIBE);
 
-if (isset($_POST["subscribe"]) && is_email($_POST["email"]) && $listsok
-   && $allthere && $validhost) {
-  $history_entry = '';
-  # make sure to save the correct data
-  if ($subscribepagedata["htmlchoice"] == "checkfortext" && !$textemail) {
-    $htmlemail = 1;
-  } else {
-    $htmlemail = isset($_POST["htmlemail"]) && $_POST["htmlemail"];
-  }
+if (isset($_POST["subscribe"]) && is_email($_POST["email"]) && $listsok && $allthere && $validhost) {
+   $history_entry = '';
+   # make sure to save the correct data
+   if ($subscribepagedata["htmlchoice"] == "checkfortext" && !$textemail) {
+      $htmlemail = 1;
+   }
+   else {
+      $htmlemail = isset($_POST["htmlemail"]) && $_POST["htmlemail"];
+   }
 
-  # now check whether this user already exists.
-  $email = $_POST["email"];
-  if (preg_match("/(.*)\n/U",$email,$regs)) {
-    $email = $regs[1];
-  }
-  $result = Sql_query("select * from {$GLOBALS["tables"]["user"]} where email = \"$email\"");#"
-  if (isset($_POST['rssfrequency'])) {
-    $rssfrequency = validateRssFrequency($_POST['rssfrequency']);
-  } else {
-    $rssfrequency = '';
-  }
+   # now check whether this user already exists.
+   $email = $_POST["email"];
+   if (preg_match("/(.*)\n/U",$email,$regs)) {
+      $email = $regs[1];
+   }
 
-  if (!Sql_affected_rows()) {
-    # they do not exist, so add them
-    $query = sprintf('insert into %s (email,entered,uniqid,confirmed,
+   $result = Sql_query("select * from {$GLOBALS["tables"]["user"]} where email = \"$email\"");#"
+
+   if (isset($_POST['rssfrequency'])) {
+      $rssfrequency = validateRssFrequency($_POST['rssfrequency']);
+   }
+   else {
+      $rssfrequency = '';
+   }
+
+   if (!Sql_affected_rows()) {
+      # they do not exist, so add them
+      $query = sprintf('insert into %s (email,entered,uniqid,confirmed,
       htmlemail,subscribepage,rssfrequency) values("%s",now(),"%s",0,%d,%d,"%s")',
-    $GLOBALS["tables"]["user"],addslashes($email),getUniqid(),$htmlemail,$id,
-    $rssfrequency);
-    $result = Sql_query($query);
-    $userid = Sql_Insert_Id();
-    addSubscriberStatistics('total users',1);
-  } else {
-    # they do exist, so update the existing record
-    # read the current values to compare changes
-    $old_data = Sql_fetch_array($result);
-    if (ASKFORPASSWORD && $old_data["password"]) {
+      $GLOBALS["tables"]["user"],addslashes($email),getUniqid(),$htmlemail,$id,
+      $rssfrequency);
+      $result = Sql_query($query);
+      $userid = Sql_Insert_Id();
+      addSubscriberStatistics('total users',1);
+   }
+   else {
+      # they do exist, so update the existing record
+      # read the current values to compare changes
+      $old_data = Sql_fetch_array($result);
+
+      /** mantis 15300
+       * when re-subscribing just update the details and don't require the password to 
+       * be the one on file
+       *
+
+      if (ASKFORPASSWORD && $old_data["password"]) {
+         if (ENCRYPTPASSWORD) {
+            $canlogin = md5($_POST["password"]) == $old_data["password"];
+         }
+         else {
+            $canlogin = $_POST["password"] == $old_data["password"];
+         }
+
+         if (!$canlogin) {
+            $msg = $GLOBALS["strUserExists"];
+            $msg.= '<p>'.$GLOBALS["strUserExistsExplanationStart"].
+            sprintf('<a href="%s&email=%s">%s</a>',getConfig("preferencesurl"),$email,
+            $GLOBALS["strUserExistsExplanationLink"]).
+            $GLOBALS["strUserExistsExplanationEnd"];
+
+            return;
+         }
+      }
+      */
+
+      $userid = $old_data["id"];
+      $old_data = array_merge($old_data,getUserAttributeValues('',$userid));
+      $history_entry = 'http://'.getConfig("website").$GLOBALS["adminpages"].'/?page=user&id='.$userid."\n\n";
+
+      $query = sprintf('update %s set email = "%s",htmlemail = %d,subscribepage = %d,rssfrequency = "%s" where id = %d',$GLOBALS["tables"]["user"],addslashes($email),$htmlemail,$id,$rssfrequency,$userid);
+      $result = Sql_query($query);
+   }
+
+   if (ASKFORPASSWORD && $_POST["password"]) {
       if (ENCRYPTPASSWORD) {
-        $canlogin = md5($_POST["password"]) == $old_data["password"];
-      } else {
-        $canlogin = $_POST["password"] == $old_data["password"];
+         $newpassword = sprintf('%s',md5($_POST["password"]));
       }
-      if (!$canlogin) {
-        $msg = $GLOBALS["strUserExists"];
-        $msg .= '<p>'.$GLOBALS["strUserExistsExplanationStart"].
-          sprintf('<a href="%s&email=%s">%s</a>',getConfig("preferencesurl"),$email,
-          $GLOBALS["strUserExistsExplanationLink"]).
-          $GLOBALS["strUserExistsExplanationEnd"];
-        return;
+      else {
+         $newpassword = sprintf('%s',$_POST["password"]);
       }
-    }
+      # see whether is has changed
 
-    $userid = $old_data["id"];
-    $old_data = array_merge($old_data,getUserAttributeValues('',$userid));
-    $history_entry = 'http://'.getConfig("website").$GLOBALS["adminpages"].'/?page=user&id='.$userid."\n\n";
+      $curpwd = Sql_Fetch_Row_Query("select password from {$GLOBALS["tables"]["user"]} where id = $userid");
 
-    $query = sprintf('update %s set email = "%s",htmlemail = %d,subscribepage = %d,rssfrequency = "%s" where id = %d',$GLOBALS["tables"]["user"],addslashes($email),$htmlemail,$id,$rssfrequency,$userid);
-    $result = Sql_query($query);
-  }
-
-  if (ASKFORPASSWORD && $_POST["password"]) {
-    if (ENCRYPTPASSWORD) {
-      $newpassword = sprintf('%s',md5($_POST["password"]));
-     } else {
-      $newpassword = sprintf('%s',$_POST["password"]);
-    }
-     # see whether is has changed
-    $curpwd = Sql_Fetch_Row_Query("select password from {$GLOBALS["tables"]["user"]} where id = $userid");
-    if ($_POST["password"] != $curpwd[0]) {
-      $storepassword = 'password = "'.$newpassword.'"';
-      Sql_query("update {$GLOBALS["tables"]["user"]} set passwordchanged = now(),$storepassword where id = $userid");
-    } else {
+      if ($_POST["password"] != $curpwd[0]) {
+         $storepassword = 'password = "'.$newpassword.'"';
+         Sql_query("update {$GLOBALS["tables"]["user"]} set passwordchanged = now(),$storepassword where id = $userid");
+      }
+      else {
+         $storepassword = "";
+      }
+   }
+   else {
       $storepassword = "";
-    }
-  } else {
-    $storepassword = "";
-  }
+   }
 
-  # subscribe to the lists
-  $lists = '';
-  if (isset($_POST['list']) && is_array($_POST["list"])) {
-    while(list($key,$val)= each($_POST["list"])) {
-      if ($val == "signup") {
-        $result = Sql_query("replace into {$GLOBALS["tables"]["listuser"]} (userid,listid,entered) values($userid,$key,now())");
-        $lists .= "\n  * ".listname($key);
-        addSubscriberStatistics('subscribe',1,$key);
+   # subscribe to the lists
+   $lists = '';
+
+   if (isset($_POST['list']) && is_array($_POST["list"])) {
+      while(list($key,$val)= each($_POST["list"])) {
+         if ($val == "signup") {
+            $key = sprintf('%d',$key);
+            if (!empty($key)) {
+              $result = Sql_query(sprintf('replace into %s (userid,listid,entered) values(%d,%d,now())',$GLOBALS["tables"]["listuser"],$userid,$key));
+              $lists .= "\n  * ".listname($key);
+
+              addSubscriberStatistics('subscribe',1,$key);
+            } else {
+              ## hack attempt...
+              exit;
+            }
+         }
       }
-    }
-  }
+   }
 
-  # remember the users attributes
-  # make sure to only remember the ones from this subscribe page
-  $history_entry .= 'Subscribe page: '.$id;
-  array_push($attributes,0);
-  $attids = join_clean(',',$attributes);
-  if ($attids && $attids != "") {
-    $res = Sql_Query("select * from ".$GLOBALS["tables"]["attribute"]." where id in ($attids)");
-    while ($row = Sql_Fetch_Array($res)) {
-      $fieldname = "attribute" .$row["id"];
-      $value = $_POST[$fieldname];
-  #    if ($value != "") {
-        if (is_array($value)) {
-          $newval = array();
-          foreach ($value as $val) {
-            array_push($newval,sprintf('%0'.$checkboxgroup_storesize.'d',$val));
-          }
-          $value = join(",",$newval);
-        } elseif ($row["type"] == "date") {
-          $value = $date->getDate($fieldname);
-        } elseif ($row['type'] != 'textarea') {
-          if (preg_match("/(.*)\n/U",$value,$regs)) {
-            $value = $regs[1];
-          }
-        }
-        Sql_Query(sprintf('replace into %s (attributeid,userid,value) values("%s","%s","%s")',
-          $GLOBALS["tables"]["user_attribute"],$row["id"],$userid,$value));
-        $history_entry .= "\n".$row["name"] . ' = '.UserAttributeValue($userid,$row["id"]);
-  #    }
-    }
-  }
-  $information_changed = 0;
-  if (isset($old_data) && is_array($old_data)) {
-    $history_subject = 'Re-Subscription';
-    # when they submit a new subscribe
-    $current_data = Sql_Fetch_Array_Query(sprintf('select * from %s where id = %d',$GLOBALS["tables"]["user"],$userid));
-    $current_data = array_merge($current_data,getUserAttributeValues('',$userid));
-    foreach ($current_data as $key => $val) {
-      if (!is_numeric($key))
-      if ($old_data[$key] != $val && $key != "password" && $key != "modified") {
-        $information_changed = 1;
-        $history_entry .= "\n$key = $val\n*changed* from $old_data[$key]";
-       }
-    }
-    if (!$information_changed) {
-      $history_entry .= "\nNo user details changed";
-    }
-  } else {
-    $history_subject = 'Subscription';
-  }
+   # remember the users attributes
+   # make sure to only remember the ones from this subscribe page
+   $history_entry .= 'Subscribe page: '.$id;
+   array_push($attributes,0);
+   $attids = join_clean(',',$attributes);
 
-  $history_entry .= "\n\nList Membership: \n$lists\n";
+   if ($attids && $attids != "") {
+      $res = Sql_Query("select * from ".$GLOBALS["tables"]["attribute"]." where id in ($attids)");
 
-  $subscribemessage = ereg_replace('\[LISTS\]', $lists, getUserConfig("subscribemessage:$id",$userid));
+      while ($row = Sql_Fetch_Array($res)) {
+         $fieldname = "attribute" .$row["id"];
+         $value = $_POST[$fieldname];
+    #    if ($value != "") {
 
-  $blacklisted = isBlackListed($email);
+         if (is_array($value)) {
+            $newval = array();
 
-  print '<title>'.$GLOBALS["strSubscribeTitle"].'</title>';
-  print $subscribepagedata["header"];
-  if (isset($_SESSION["adminloggedin"]) && $_SESSION["adminloggedin"]) {
-    print '<p><b>You are logged in as '.$_SESSION["logindetails"]["adminname"].'</b></p>';
-    print '<p><a href="'.$adminpages.'">Back to the main admin page</a></p>';
-    if ($_POST["makeconfirmed"] && !$blacklisted) {
-      $sendrequest = 0;
-      Sql_Query(sprintf('update %s set confirmed = 1 where email = "%s"',$GLOBALS["tables"]["user"],$email));
-      addUserHistory($email,$history_subject." by ".$_SESSION["logindetails"]["adminname"],$history_entry);
-    } elseif ($_POST["makeconfirmed"]) {
-      print '<p>'.$GLOBALS['I18N']->get('Email is blacklisted, so request for confirmation has been sent.').'<br/>';
-      print $GLOBALS['I18N']->get('If user confirms subscription, they will be removed from the blacklist.').'</p>';
+            foreach ($value as $val) {
+               array_push($newval,sprintf('%0'.$checkboxgroup_storesize.'d',$val));
+            }  
+
+            $value = join(",",$newval);
+         }
+         elseif ($row["type"] == "date") {
+            $value = $date->getDate($fieldname);
+         }
+         elseif ($row['type'] != 'textarea') {
+            if (preg_match("/(.*)\n/U",$value,$regs)) {
+               $value = $regs[1];
+            }
+         } 
+
+         Sql_Query(sprintf('replace into %s (attributeid,userid,value) values("%s","%s","%s")',
+
+         $GLOBALS["tables"]["user_attribute"],$row["id"],$userid,$value));
+         $history_entry .= "\n".$row["name"] . ' = '.UserAttributeValue($userid,$row["id"]);
+    #    }
+      }
+   }
+
+   $information_changed = 0;
+
+   if (isset($old_data) && is_array($old_data)) {
+      $history_subject = 'Re-Subscription';
+      # when they submit a new subscribe
+      $current_data = Sql_Fetch_Array_Query(sprintf('select * from %s where id = %d',$GLOBALS["tables"]["user"],$userid));
+      $current_data = array_merge($current_data,getUserAttributeValues('',$userid));
+      foreach ($current_data as $key => $val) {
+         if (!is_numeric($key))
+            if ($old_data[$key] != $val && $key != "password" && $key != "modified") {
+               $information_changed = 1;
+               $history_entry .= "\n$key = $val\n*changed* from $old_data[$key]";
+            }
+      }
+      if (!$information_changed) {
+         $history_entry .= "\nNo user details changed";
+      }
+   }
+   else {
+      $history_subject = 'Subscription';
+   }
+
+   $history_entry .= "\n\nList Membership: \n$lists\n";
+
+   $subscribemessage = ereg_replace('\[LISTS\]', $lists, getUserConfig("subscribemessage:$id",$userid));
+
+   $blacklisted = isBlackListed($email);
+
+   print '<title>'.$GLOBALS["strSubscribeTitle"].'</title>';
+   print $subscribepagedata["header"];
+
+   if (isset($_SESSION["adminloggedin"]) && $_SESSION["adminloggedin"]) {
+      print '<p><b>You are logged in as '.$_SESSION["logindetails"]["adminname"].'</b></p>';
+      print '<p><a href="'.$adminpages.'">Back to the main admin page</a></p>';
+
+      if ($_POST["makeconfirmed"] && !$blacklisted) {
+         $sendrequest = 0;
+         Sql_Query(sprintf('update %s set confirmed = 1 where email = "%s"',$GLOBALS["tables"]["user"],$email));
+         addUserHistory($email,$history_subject." by ".$_SESSION["logindetails"]["adminname"],$history_entry);
+      }
+      elseif ($_POST["makeconfirmed"]) {
+         print '<p>'.$GLOBALS['I18N']->get('Email is blacklisted, so request for confirmation has been sent.').'<br/>';
+         print $GLOBALS['I18N']->get('If user confirms subscription, they will be removed from the blacklist.').'</p>';
+
+         $sendrequest = 1;
+      }
+      else {
+         $sendrequest = 1;
+      }
+   }
+   else {
       $sendrequest = 1;
-    } else {
-      $sendrequest = 1;
-    }
-  } else {
-    $sendrequest = 1;
-  }
+   }
 
-  # personalise the thank you page
-  if ($subscribepagedata["thankyoupage"]) {
-    $thankyoupage = $subscribepagedata["thankyoupage"];
-  } else {
-    $thankyoupage = '<h3>'.$strThanks.'</h3>'. $strEmailConfirmation;
-  }
+   # personalise the thank you page
+   if ($subscribepagedata["thankyoupage"]) {
+      $thankyoupage = $subscribepagedata["thankyoupage"];
+   }
+   else {
+      $thankyoupage = '<h3>'.$strThanks.'</h3>'. $strEmailConfirmation;
+   }
 
-  if (eregi("\[email\]",$thankyoupage,$regs))
-    $thankyoupage = eregi_replace("\[email\]",$email,$thankyoupage);
-  $user_att = getUserAttributeValues($email);
-  while (list($att_name,$att_value) = each ($user_att)) {
-    if (eregi("\[".$att_name."\]",$thankyoupage,$regs))
-      $thankyoupage = eregi_replace("\[".$att_name."\]",$att_value,$thankyoupage);
-  }
+   if (eregi("\[email\]",$thankyoupage,$regs))
+   $thankyoupage = eregi_replace("\[email\]",$email,$thankyoupage);
 
-  if (is_array($GLOBALS["plugins"])) {
-    reset($GLOBALS["plugins"]);
-    foreach ($GLOBALS["plugins"] as $name => $plugin) {
-      $thankyoupage = $plugin->parseThankyou($id,$userid,$thankyoupage);
-    }
-  }
+   $user_att = getUserAttributeValues($email);
 
-  $blacklisted = isBlackListed($email);
-  if ($blacklisted) {
-    $thankyoupage .= '<p>'.$GLOBALS['I18N']->get('YouAreBlacklisted').'</p>';
-  }
-  if ($sendrequest && $listsok) { #is_array($_POST["list"])) {
-    if (sendMail($email, getConfig("subscribesubject:$id"), $subscribemessage,system_messageheaders($email),$envelope,1)) {
-      sendAdminCopy("Lists subscription","\n".$email . " has subscribed\n\n$history_entry");
-      addUserHistory($email,$history_subject,$history_entry);
+   while (list($att_name,$att_value) = each ($user_att)) {
+      if (eregi("\[".$att_name."\]",$thankyoupage,$regs))
+         $thankyoupage = eregi_replace("\[".$att_name."\]",$att_value,$thankyoupage);
+   }
+
+   if (is_array($GLOBALS["plugins"])) {
+      reset($GLOBALS["plugins"]);
+
+      foreach ($GLOBALS["plugins"] as $name => $plugin) {
+         $thankyoupage = $plugin->parseThankyou($id,$userid,$thankyoupage);
+      }
+   }
+
+   # $blacklisted = isBlackListed($email);
+   # 
+   # if ($blacklisted) {
+   #    $thankyoupage .= '<p>'.$GLOBALS["strYouAreBlacklisted"].'</p>';
+   #    return 1; // commented out to remove the second window bug - issues 15240 & 15290
+   # }
+
+   if ($sendrequest && $listsok) { #is_array($_POST["list"])) {
+      if (sendMail($email, getConfig("subscribesubject:$id"), $subscribemessage,system_messageheaders($email),'',1)) {
+         sendAdminCopy("Lists subscription","\n".$email . " has subscribed\n\n$history_entry");
+         addUserHistory($email,$history_subject,$history_entry);
+         print $thankyoupage;
+      }
+      else {
+         print '<h3>'.$strEmailFailed.'</h3>';
+
+         if ($blacklisted) {
+            print '<p>'.$GLOBALS["strYouAreBlacklisted"].'</p>';
+         }
+      }
+   }
+   else {
       print $thankyoupage;
-     } else {
-      print '<h3>'.$strEmailFailed.'</h3>';
-      if ($blacklisted) {
-        print '<p>'.$GLOBALS['I18N']->get('YouAreBlacklisted').'</p>';
-      }
-    }
-  } else {
-    print $thankyoupage;
-    if ($_SESSION["adminloggedin"]) {
-      print "<p>User has been added and confirmed</p>";
-    }
-  }
 
-  print "<P>".$PoweredBy.'</p>';
-  print $subscribepagedata["footer"];
-//  exit;
-  // Instead of exiting here, we return 2. So in lists/index.php
-  // We can decide, whether to show subcribe page or not.
-  ## issue 6508
-  return 2;
-} elseif (isset($_POST["update"]) && $_POST["update"] && is_email($_POST["email"]) && $allthere) {
+      if ($_SESSION["adminloggedin"]) {
+         print "<p>User has been added and confirmed</p>";
+      }
+   }
+
+   print "<P>".$PoweredBy.'</p>';
+   print $subscribepagedata["footer"];
+   //  exit;
+   // Instead of exiting here, we return 2. So in lists/index.php
+   // We can decide, whether to show subcribe page or not.
+   ## issue 6508
+   return 2;
+}
+elseif (isset($_POST["update"]) && $_POST["update"] && is_email($_POST["email"]) && $allthere) {
   $email = trim($_POST["email"]);
   if (preg_match("/(.*)\n/U",$email,$regs)) {
     $email = $regs[1];
@@ -482,7 +564,7 @@ if (isset($_POST["subscribe"]) && is_email($_POST["email"]) && $listsok
         Sql_query(sprintf('replace into %s (attributeid,userid,value) values("%s","%s","%s")',
           $GLOBALS["tables"]["user_attribute"],$attribute["id"],$userid,$value));
         if ($attribute["type"] != "hidden") {
-          $datachange .= strip_tags($attribute["name"]) . " = ";
+          $datachange .= strip_tags($attribute["name"]) . " : ";
           if ($attribute["type"] == "checkbox")
             $datachange .= $value?$strYes:$strNo;
           elseif ($attribute["type"] != "date" && $attribute["type"] != "textline" && $attribute["type"] != "textarea")
@@ -521,8 +603,8 @@ if (isset($_POST["subscribe"]) && is_email($_POST["email"]) && $listsok
   print $subscribepagedata["header"];
   if (!TEST) {
     if ($emailchanged) {
-      if (sendMail($data["email"],getConfig("updatesubject"),$oldaddressmessage, system_messageheaders($email),$envelope) &&
-        sendMail($email,getConfig("updatesubject"),$newaddressmessage, system_messageheaders($email),$envelope)) {
+      if (sendMail($data["email"],getConfig("updatesubject"),$oldaddressmessage, system_messageheaders($email),'') &&
+        sendMail($email,getConfig("updatesubject"),$newaddressmessage, system_messageheaders($email),'')) {
         $ok = 1;
         sendAdminCopy("Lists information changed","\n".$data["email"] . " has changed their information.\n\nThe email has changed to $email.\n\n$history_entry");
         addUserHistory($email,"Change",$history_entry);
@@ -530,7 +612,7 @@ if (isset($_POST["subscribe"]) && is_email($_POST["email"]) && $listsok
         $ok = 0;
       }
     } else {
-      if (sendMail($email, getConfig("updatesubject"), $message, system_messageheaders($email),$envelope)) {
+      if (sendMail($email, getConfig("updatesubject"), $message, system_messageheaders($email),'')) {
         $ok = 1;
         sendAdminCopy("Lists information changed","\n".$data["email"] . " has changed their information\n\n$history_entry");
         addUserHistory($email,"Change",$history_entry);
@@ -553,7 +635,7 @@ if (isset($_POST["subscribe"]) && is_email($_POST["email"]) && $listsok
   print "<P>".$PoweredBy.'</p>';
   print $subscribepagedata["footer"];
   // exit;
-  // Instead of exiting here, we return 2. So in lists/index.php
+  // Instead of exiting here, we return 3. So in lists/index.php
   // We can decide, whether to show preferences page or not.
   ## mantis issue 6508
   return 3; 
@@ -561,10 +643,10 @@ if (isset($_POST["subscribe"]) && is_email($_POST["email"]) && $listsok
   $msg = '<div class="missing">'.$strEnterEmail.'</div><br/>';
 } elseif ((isset($_POST["subscribe"]) || isset($_POST["update"])) && !$validhost) {
   $msg = '<div class="missing">'.$strInvalidHostInEmail.'</div><br/>';
-} elseif ((isset($_POST["subscribe"]) || isset($_POST["update"])) && !isset($_POST["list"]) && !ALLOW_NON_LIST_SUBSCRIBE) {
-  $msg = '<div class="missing">'.$strEnterList.'</div><br/>';
 } elseif ((isset($_POST["subscribe"]) || isset($_POST["update"])) && $missing) {
   $msg = '<div class="missing">'."$strValuesMissing: $missing".'</div><br/>';
+} elseif ((isset($_POST["subscribe"]) || isset($_POST["update"])) && !isset($_POST["list"]) && !ALLOW_NON_LIST_SUBSCRIBE) {
+  $msg = '<div class="missing">'.$strEnterList.'</div><br/>';
 } else {
 #  $msg = 'Unknown Error';
 }
@@ -666,6 +748,7 @@ function ListAttributes($attributes,$attributedata,$htmlchoice = 0,$userid = 0,$
   if (!$textarearows) $textarearows = 10;
   if (!$textareacols) $textareacols = 40;
 
+  $html = '';
   if (!isset($_GET['page']) || (isset($_GET['page']) && $_GET["page"] != "import1"))
   $html = sprintf('
   <tr><td><div class="required">%s</div></td>
@@ -681,7 +764,7 @@ $html .= sprintf('
   <tr><td><div class="required">%s</div></td>
   <td class="attributeinput"><input type=text name=emailconfirm value="%s" size="%d">
   <script language="Javascript" type="text/javascript">addFieldToCheck("emailconfirm","%s");</script></td></tr>',
- 'Confirm email',htmlspecialchars(stripslashes($_REQUEST["emailconfirm"])),$textlinewidth,'Confirm email');
+  $GLOBALS["strConfirmEmail"],htmlspecialchars(stripslashes($_REQUEST["emailconfirm"])),$textlinewidth, $GLOBALS["strConfirmEmail"]);
 }
 // BPM 12 May 2004 - Finish
 
@@ -709,6 +792,7 @@ $html .= sprintf('
   $pwdclass,$GLOBALS["strPassword2"],$textlinewidth,$js2);
    }
 
+## Write attribute fields
   switch($htmlchoice) {
     case "textonly":
       if (!isset($htmlemail))
@@ -797,12 +881,14 @@ $html .= sprintf('
             $checked = $data[$attr["id"]] ? "checked":"";
           $output[$attr["id"]] .= sprintf("\n".'<input type="checkbox" name="%s" value="on" %s class="attributeinput">',$fieldname,$checked);
           $output[$attr["id"]] .= sprintf("\n".'<span class="%s">%s</span>',$attr["required"] ? 'required' : 'attributename',stripslashes($attr["name"]));
+          if ($attr["required"])
+            $output[$attr["id"]] .= sprintf('<script language="Javascript" type="text/javascript">addFieldToCheck("%s","%s");</script>',$fieldname,$attr["name"]);
           break;
         case "radio":
           $output[$attr["id"]] .= sprintf("\n".'<tr><td colspan=2><div class="%s">%s</div>',$attr["required"] ? 'required' : 'attributename',stripslashes($attr["name"]));
           $values_request = Sql_Query("select * from $table_prefix"."listattr_".$attr["tablename"]." order by listorder,name");
           while ($value = Sql_Fetch_array($values_request)) {
-            if ($_POST[$fieldname])
+            if (!empty($_POST[$fieldname]))
               $checked = $_POST[$fieldname] == $value["id"] ? "checked":"";
             else if ($data[$attr["id"]])
               $checked = $data[$attr["id"]] == $value["id"] ? "checked":"";
@@ -811,13 +897,15 @@ $html .= sprintf('
             $output[$attr["id"]] .= sprintf('&nbsp;%s&nbsp;<input type=radio  class="attributeinput" name="%s" value="%s" %s>',
               $value["name"],$fieldname,$value["id"],$checked);
           }
+          if ($attr["required"])
+            $output[$attr["id"]] .= sprintf('<script language="Javascript" type="text/javascript">addGroupToCheck("%s","%s");</script>',$fieldname,$attr["name"]);
           break;
         case "select":
           $output[$attr["id"]] .= sprintf("\n".'<tr><td><div class="%s">%s</div>',$attr["required"] ? 'required' : 'attributename',stripslashes($attr["name"]));
           $values_request = Sql_Query("select * from $table_prefix"."listattr_".$attr["tablename"]." order by listorder,name");
           $output[$attr["id"]] .= sprintf('</td><td class="attributeinput"><!--%d--><select name="%s" class="attributeinput">',$data[$attr["id"]],$fieldname);
           while ($value = Sql_Fetch_array($values_request)) {
-            if ($_POST[$fieldname])
+            if (!empty($_POST[$fieldname]))
               $selected = $_POST[$fieldname] == $value["id"] ? "selected" : "";
             else if ($data[$attr["id"]])
               $selected = $data[$attr["id"]] == $value["id"] ? "selected":"";
@@ -841,6 +929,11 @@ $html .= sprintf('
               $selection = explode(",",$data[$attr["id"]]);
               $selected = in_array($value["id"],$selection) ? "checked":"";
             }
+            else{
+               $selection = Array();
+               $selected  = "";
+            }
+
             $output[$attr["id"]] .= sprintf('<tr><td colspan=2 class="attributeinput"><input type=checkbox name="%s[]"  class="attributeinput" value="%s" %s> %s</td></tr>',$fieldname,$value["id"],$selected,stripslashes($value["name"]));
           }
           break;
